@@ -13,7 +13,6 @@
 #include <sstream>
 #include <memory>
 #include <iomanip>
-#include <codecvt>
 
 #include "keyman_core.h"
 #include "jsonpp.hpp"
@@ -22,6 +21,7 @@
 #include "state.hpp"
 #include "vkey_to_contextreset.hpp"
 #include "kmx_file.h"
+#include "utfcodec.hpp"
 
 using namespace km::core;
 
@@ -225,7 +225,7 @@ km_core_status km_core_state_to_json(km_core_state const *state,
   {
     // Pretty print the document.
     jo << json::object
-        << "$schema" << "keyman/core/doc/introspection.schema"
+        << "$schema" << "keyman/core/docs/introspection.schema"
         << "keyboard" << state->processor().keyboard()
 //        << "options" << state->options()  TODO: Fix
         << "context" << state->context()
@@ -285,22 +285,22 @@ km_core_status km_core_state_context_clear(
   return KM_CORE_STATUS_OK;
 }
 
-void km_core_cp_dispose(
-  km_core_cp *cp
+void km_core_cu_dispose(
+  km_core_cu *cp
 ) {
   if(cp != nullptr) {
     delete [] cp;
   }
 }
 
-km_core_cp * _new_error_string(std::u16string const str) {
-  km_core_cp* result = new km_core_cp[str.size()+1];
+km_core_cu * _new_error_string(std::u16string const str) {
+  km_core_cu* result = new km_core_cu[str.size()+1];
   str.copy(result, str.size());
   result[str.size()] = 0;
   return result;
 }
 
-km_core_cp * km_core_state_context_debug(
+km_core_cu * km_core_state_context_debug(
   km_core_state *state,
   km_core_debug_context_type context_type
 ) {
@@ -357,9 +357,9 @@ km_core_cp * km_core_state_context_debug(
 
   km_core_context_items_dispose(context_items);
 
-  std::u16string s = std::wstring_convert<std::codecvt_utf8_utf16<char16_t>, char16_t>{}.from_bytes(buffer.str());
+  std::u16string s = convert<char, char16_t>(buffer.str());
 
-  km_core_cp* result = new km_core_cp[s.size() + 1];
+  km_core_cu* result = new km_core_cu[s.size() + 1];
   s.copy(result, s.size());
   result[s.size()] = 0;
 
@@ -385,11 +385,11 @@ state_should_invalidate_context(km_core_state *state,
   // if emit_keystroke is present, check if a context reset is needed
   if (state_has_action_type(state, KM_CORE_IT_EMIT_KEYSTROKE)) {
     if (
-        // when a backspace keystroke is emitted, it is because we are at the start of 
-        // context, and we want to give the application the chance to process it, e.g. 
-        // by moving to previous field. Note that context manipulation does not result 
-        // in an emit_keystroke backspace action, as this is handled through the 
-        // `code_points_to_delete` field. So we always invalidate context when a 
+        // when a backspace keystroke is emitted, it is because we are at the start of
+        // context, and we want to give the application the chance to process it, e.g.
+        // by moving to previous field. Note that context manipulation does not result
+        // in an emit_keystroke backspace action, as this is handled through the
+        // `code_points_to_delete` field. So we always invalidate context when a
         // processor emits a backspace.
         vk == KM_CORE_VKEY_BKSP ||
         // certain modifiers invalidate context
