@@ -232,7 +232,7 @@ get_context_debug(IBusEngine *engine) {
 static gchar *
 debug_utf8_with_codepoints(const gchar *utf8) {
   GString *output = g_string_new("");
-  g_string_append_printf(output, "|%s| len:%ld) [", utf8, g_utf8_strlen(utf8, -1));
+  g_string_append_printf(output, "|%s| (len:%ld) [", utf8, g_utf8_strlen(utf8, -1));
   gunichar2 *utf16 = g_utf8_to_utf16(utf8, -1, NULL, NULL, NULL);
   for (int i = 0; utf16[i] != '\0'; i++) {
     g_string_append_printf(output, "U+%04x ", utf16[i]);
@@ -290,7 +290,7 @@ set_context_if_needed(IBusEngine *engine) {
 }
 
 static void
-initialize_queue(IBusKeymanEngine *keyman, int index, int count) {
+initialize_queue_items(IBusKeymanEngine *keyman, int index, int count) {
   g_assert(keyman != NULL);
   g_assert(index >= 0 && index < MAX_QUEUE_SIZE);
   g_assert(count > 0 && count <= MAX_QUEUE_SIZE);
@@ -463,7 +463,7 @@ ibus_keyman_engine_constructor(
     keyman->lctrl_pressed = FALSE;
     keyman->ralt_pressed = FALSE;
     keyman->rctrl_pressed = FALSE;
-    initialize_queue(keyman, 0, MAX_QUEUE_SIZE);
+    initialize_queue_items(keyman, 0, MAX_QUEUE_SIZE);
     keyman->commit_item    = &keyman->commit_queue[0];
     gchar **split_name     = g_strsplit(engine_name, ":", 2);
     if (split_name[0] == NULL)
@@ -608,6 +608,14 @@ ibus_keyman_engine_destroy (IBusKeymanEngine *keyman)
     IBUS_OBJECT_CLASS (parent_class)->destroy ((IBusObject *)keyman);
 }
 
+/**
+ * Set text on the client
+ *
+ * Called by the Keyman service from the handler of the `SendText` method.
+ *
+ * @param engine  A pointer to the IBusEngine instance
+ * @param text    The text to be set on the client
+ */
 void ibus_keyman_set_text(IBusEngine *engine, const gchar *text)
 {
     IBusKeymanEngine *keyman = (IBusKeymanEngine *)engine;
@@ -701,20 +709,25 @@ process_persist_action(IBusEngine *engine, km_core_option_item *persist_options)
   }
 }
 
-static void
+/**
+ * Process the emit_keystroke action
+ *
+ * @param engine          A pointer to the IBusEngine instance.
+ * @param emit_keystroke  A boolean indicating whether to emit a keystroke.
+ *
+ * @returns TRUE if Keyman handled the event, FALSE otherwise.
+ */
+static gboolean
 process_emit_keystroke_action(IBusEngine *engine, km_core_bool emit_keystroke) {
-  if (!emit_keystroke) {
-    return;
-  }
   IBusKeymanEngine *keyman = (IBusKeymanEngine *)engine;
   if (client_supports_surrounding_text(engine)) {
     // compliant app
-    ibus_engine_forward_key_event(engine, keyman->commit_item->keyval,
-      keyman->commit_item->keycode, keyman->commit_item->state);
-    return;
+    return !emit_keystroke;
   }
+
   // non-compliant app
-  keyman->commit_item->emitting_keystroke = TRUE;
+  keyman->commit_item->emitting_keystroke = emit_keystroke;
+  return TRUE;
 }
 
 static void
@@ -768,7 +781,7 @@ commit_current_queue_item(IBusKeymanEngine *keyman) {
   }
   keyman->commit_item--;
   memmove(keyman->commit_queue, &keyman->commit_queue[1], sizeof(commit_queue_item) * MAX_QUEUE_SIZE - 1);
-  initialize_queue(keyman, MAX_QUEUE_SIZE - 1, 1);
+  initialize_queue_items(keyman, MAX_QUEUE_SIZE - 1, 1);
 }
 
 static void
@@ -812,7 +825,15 @@ finish_process_actions(IBusEngine *engine) {
   }
 }
 
-static void
+/**
+ * Process the actions from Keyman Core
+ *
+ * @param engine   A pointer to the IBusEngine instance.
+ * @param actions  A pointer to the km_core_actions structure containing the actions to process.
+ *
+ * @returns TRUE if Keyman handled the event, FALSE otherwise.
+ */
+static gboolean
 process_actions(
   IBusEngine *engine,
   km_core_actions const *actions
@@ -821,11 +842,29 @@ process_actions(
   process_output_action(engine, actions->output);
   process_persist_action(engine, actions->persist_options);
   process_alert_action(actions->do_alert);
-  process_emit_keystroke_action(engine, actions->emit_keystroke);
+  gboolean result = process_emit_keystroke_action(engine, actions->emit_keystroke);
   process_capslock_action(actions->new_caps_lock_state);
   finish_process_actions(engine);
+  return result;
 }
 
+/**
+ * Handler for the IBus `process-key-event` signal
+ *
+ * Called by IBus when the user presses a key. Called twice for each
+ * keypress: once when the key is pressed down and once when it is
+ * released.
+ *
+ * @param engine   A pointer to the IBusEngine instance
+ * @param keyval   The key value of the keypress
+ * @param keycode  The key code of the keypress
+ * @param state    Modifier flags
+ *
+ * @return TRUE if we processed the key event, i.e. if there were
+ * matching rules in the keyboard. No further processing of this key event
+ * will be done. Otherwise FALSE, allowing other interested parties to
+ * respond to the key event.
+ */
 static gboolean
 ibus_keyman_engine_process_key_event(
   IBusEngine *engine,
@@ -956,25 +995,41 @@ ibus_keyman_engine_process_key_event(
   keyman->commit_item->char_buffer = NULL;
   const km_core_actions *core_actions = km_core_state_get_actions(keyman->state);
 
-  process_actions(engine, core_actions);
+  gboolean result = process_actions(engine, core_actions);
 
   // If we have a non-compliant client, i.e. a client that doesn't support
   // surrounding text (e.g. Chromium as of v104) we sent the key event
   // to the system service and now stop further processing by returning TRUE.
-  // With a compliant client (i.e. it does support surrounding text), we return
-  // TRUE because we completely processed the event and no further
-  // processing should happen.
-  g_message("%s: after processing all actions: %s", __FUNCTION__, debug_context2 = get_context_debug(engine));
+  //
+  // With a compliant client (i.e. supports surrounding text), it
+  // depends whether or not we handled the event. We return TRUE if we
+  // completely processed the event and no further processing should happen,
+  // or FALSE if someone else should handle the event (e.g. for a left-arrow
+  // key event).
+  g_message("%s: after processing all actions: %s, returning %s", __FUNCTION__,
+    debug_context2 = get_context_debug(engine), result ? "TRUE" : "FALSE");
 
-  return TRUE;
+  return result;
 }
 
+/**
+ * Handler for the IBus `set-surrounding-text` signal
+ *
+ * Called by IBus to let us know the surrounding text and the cursor position
+ * (and selection) in the text. This is used by Keyman as context.
+ *
+ * @param engine      A pointer to the IBusEngine instance
+ * @param text        The surrounding text
+ * @param cursor_pos  The position of the cursor in the text
+ * @param anchor_pos  The position of the anchor in the text
+ */
 static void
-ibus_keyman_engine_set_surrounding_text (IBusEngine *engine,
-                                            IBusText    *text,
-                                            guint       cursor_pos,
-                                            guint       anchor_pos)
-{
+ibus_keyman_engine_set_surrounding_text(
+  IBusEngine *engine,
+  IBusText    *text,
+  guint       cursor_pos,
+  guint       anchor_pos
+){
     parent_class->set_surrounding_text(engine, text, cursor_pos, anchor_pos);
     set_context_if_needed(engine);
 }
@@ -990,6 +1045,13 @@ ibus_keyman_engine_set_surrounding_text (IBusEngine *engine,
 //     parent_class->set_cursor_location (engine, x, y, w, h);
 // }
 
+/**
+ * Handler for the IBus `focus-in` signal
+ *
+ * Called by ibus when the client application receives focus
+ *
+ * @param engine  A pointer to the IBusEngine instance
+ */
 static void
 ibus_keyman_engine_focus_in (IBusEngine *engine)
 {
@@ -1002,6 +1064,13 @@ ibus_keyman_engine_focus_in (IBusEngine *engine)
     parent_class->focus_in (engine);
 }
 
+/**
+ * Handler for the IBus `focus-out` signal
+ *
+ * Called by ibus when the client application loses focus
+ *
+ * @param engine  A pointer to the IBusEngine instance
+ */
 static void
 ibus_keyman_engine_focus_out (IBusEngine *engine)
 {
@@ -1012,6 +1081,13 @@ ibus_keyman_engine_focus_out (IBusEngine *engine)
     parent_class->focus_out (engine);
 }
 
+/**
+ * Handler for the IBus `reset` signal
+ *
+ * Called by ibus when the IME is reset.
+ *
+ * @param engine  A pointer to the IBusEngine instance
+ */
 static void
 ibus_keyman_engine_reset (IBusEngine *engine)
 {
@@ -1020,8 +1096,13 @@ ibus_keyman_engine_reset (IBusEngine *engine)
     set_context_if_needed(engine);
 }
 
-
-
+/**
+ * Handler for the IBus `enable` signal
+ *
+ * Called by ibus when the IME is enabled.
+ *
+ * @param engine  A pointer to the IBusEngine instance
+ */
 static void
 ibus_keyman_engine_enable (IBusEngine *engine)
 {
@@ -1043,6 +1124,13 @@ ibus_keyman_engine_enable (IBusEngine *engine)
     parent_class->enable (engine);
 }
 
+/**
+ * Handler for the IBus `disable` signal
+ *
+ * Called by ibus when the IME is disabled.
+ *
+ * @param engine  A pointer to the IBusEngine instance
+ */
 static void
 ibus_keyman_engine_disable (IBusEngine *engine)
 {
